import os
import shutil
import tempfile
import unittest

from blueberrypy.project import create_project

class CreateProjectTest(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_create_project(self):

        blueberrypy_config = {'author': 'Jimmy Wong',
                              'current_year': 2012,
                              'driver': 'pysqlite',
                              'path': self.tmpdir,
                              'email': 'abc@example.com',
                              'package': 'blueberrytest',
                              'project_name': 'blueberrytest',
                              'sqlalchemy_url': 'sqlite://',
                              'use_controller': True,
                              'use_jinja2': True,
                              'use_redis': True,
                              'use_rest_controller': True,
                              'use_sqlalchemy': True,
                              'use_webassets': True,
                              'version': '0.1'}

        create_project(blueberrypy_config)

        paths = ['',
                 '.cache',
                 'config',
                 'config/dev',
                 'config/dev/app.yml',
                 'config/dev/bundles.yml',
                 'config/dev/logging.yml',
                 'config/prod',
                 'config/prod/app.yml',
                 'config/prod/bundles.yml',
                 'config/prod/logging.yml',
                 'log',
                 'setup.py',
                 'src',
                 'src/blueberrytest',
                 'src/blueberrytest/__init__.py',
                 'src/blueberrytest/api.py',
                 'src/blueberrytest/controller.py',
                 'src/blueberrytest/model.py',
                 'src/blueberrytest/rest_controller.py',
                 'src/blueberrytest/templates',
                 'src/blueberrytest/templates/404.html',
                 'src/blueberrytest/templates/base.html',
                 'src/blueberrytest/templates/index.html',
                 'src/tests',
                 'src/tests/__init__.py',
                 'src/tests/helper.py',
                 'src/tests/test_api.py',
                 'src/tests/test_controller.py',
                 'src/tests/test_model.py',
                 'src/tests/test_rest_controller.py',
                 'static',
                 'static/css',
                 'static/css/screen.css',
                 'static/favicon.ico',
                 'static/img',
                 'static/js']

        for path in paths:
            self.assertTrue(os.path.exists(os.path.join(self.tmpdir, path)), "Path '%s' wasn't created" % path)

    def test_create_project_use_sqlalchemy_n(self):

        blueberrypy_config = {'author': 'Jimmy Wong',
                              'current_year': 2012,
                              'driver': 'pysqlite',
                              'path': self.tmpdir,
                              'email': 'abc@example.com',
                              'package': 'blueberrytest',
                              'project_name': 'blueberrytest',
                              'sqlalchemy_url': 'sqlite://',
                              'use_controller': True,
                              'use_jinja2': True,
                              'use_redis': True,
                              'use_rest_controller': True,
                              'use_sqlalchemy': False,
                              'use_webassets': True,
                              'version': '0.1'}

        create_project(blueberrypy_config)

        paths = ['',
                 '.cache',
                 'config',
                 'config/dev',
                 'config/dev/app.yml',
                 'config/dev/bundles.yml',
                 'config/dev/logging.yml',
                 'config/prod',
                 'config/prod/app.yml',
                 'config/prod/bundles.yml',
                 'config/prod/logging.yml',
                 'log',
                 'setup.py',
                 'src',
                 'src/blueberrytest',
                 'src/blueberrytest/__init__.py',
                 'src/blueberrytest/controller.py',
                 'src/blueberrytest/rest_controller.py',
                 'src/blueberrytest/templates',
                 'src/blueberrytest/templates/404.html',
                 'src/blueberrytest/templates/base.html',
                 'src/blueberrytest/templates/index.html',
                 'src/tests',
                 'src/tests/__init__.py',
                 'src/tests/helper.py',
                 'src/tests/test_controller.py',
                 'src/tests/test_rest_controller.py',
                 'static',
                 'static/css',
                 'static/css/screen.css',
                 'static/favicon.ico',
                 'static/img',
                 'static/js']

        for path in paths:
            self.assertTrue(os.path.exists(os.path.join(self.tmpdir, path)), "Path '%s' wasn't created" % path)

    def test_create_project_use_controller_n(self):

        blueberrypy_config = {'author': 'Jimmy Wong',
                              'current_year': 2012,
                              'driver': 'pysqlite',
                              'path': self.tmpdir,
                              'email': 'abc@example.com',
                              'package': 'blueberrytest',
                              'project_name': 'blueberrytest',
                              'sqlalchemy_url': 'sqlite://',
                              'use_controller': False,
                              'use_jinja2': True,
                              'use_redis': True,
                              'use_rest_controller': True,
                              'use_sqlalchemy': True,
                              'use_webassets': True,
                              'version': '0.1'}

        create_project(blueberrypy_config)

        paths = ['',
                 '.cache',
                 'config',
                 'config/dev',
                 'config/dev/app.yml',
                 'config/dev/bundles.yml',
                 'config/dev/logging.yml',
                 'config/prod',
                 'config/prod/app.yml',
                 'config/prod/bundles.yml',
                 'config/prod/logging.yml',
                 'log',
                 'setup.py',
                 'src',
                 'src/blueberrytest',
                 'src/blueberrytest/__init__.py',
                 'src/blueberrytest/api.py',
                 'src/blueberrytest/model.py',
                 'src/blueberrytest/rest_controller.py',
                 'src/tests',
                 'src/tests/__init__.py',
                 'src/tests/helper.py',
                 'src/tests/test_api.py',
                 'src/tests/test_model.py',
                 'src/tests/test_rest_controller.py']

        for path in paths:
            self.assertTrue(os.path.exists(os.path.join(self.tmpdir, path)), "Path '%s' wasn't created" % path)

    def test_create_project_use_jinja2_n(self):

        blueberrypy_config = {'author': 'Jimmy Wong',
                              'current_year': 2012,
                              'driver': 'pysqlite',
                              'path': self.tmpdir,
                              'email': 'abc@example.com',
                              'package': 'blueberrytest',
                              'project_name': 'blueberrytest',
                              'sqlalchemy_url': 'sqlite://',
                              'use_controller': True,
                              'use_jinja2': False,
                              'use_redis': True,
                              'use_rest_controller': True,
                              'use_sqlalchemy': True,
                              'use_webassets': True,
                              'version': '0.1'}

        create_project(blueberrypy_config)

        paths = ['',
                 '.cache',
                 'config',
                 'config/dev',
                 'config/dev/app.yml',
                 'config/dev/bundles.yml',
                 'config/dev/logging.yml',
                 'config/prod',
                 'config/prod/app.yml',
                 'config/prod/bundles.yml',
                 'config/prod/logging.yml',
                 'log',
                 'setup.py',
                 'src',
                 'src/blueberrytest',
                 'src/blueberrytest/__init__.py',
                 'src/blueberrytest/api.py',
                 'src/blueberrytest/controller.py',
                 'src/blueberrytest/model.py',
                 'src/blueberrytest/rest_controller.py',
                 'src/tests',
                 'src/tests/__init__.py',
                 'src/tests/helper.py',
                 'src/tests/test_api.py',
                 'src/tests/test_controller.py',
                 'src/tests/test_model.py',
                 'src/tests/test_rest_controller.py',
                 'static',
                 'static/css',
                 'static/css/screen.css',
                 'static/favicon.ico',
                 'static/img',
                 'static/js']

        for path in paths:
            self.assertTrue(os.path.exists(os.path.join(self.tmpdir, path)), "Path '%s' wasn't created" % path)

    def test_create_project_use_rest_controller_n(self):

        blueberrypy_config = {'author': 'Jimmy Wong',
                              'current_year': 2012,
                              'driver': 'pysqlite',
                              'path': self.tmpdir,
                              'email': 'abc@example.com',
                              'package': 'blueberrytest',
                              'project_name': 'blueberrytest',
                              'sqlalchemy_url': 'sqlite://',
                              'use_controller': True,
                              'use_jinja2': True,
                              'use_redis': True,
                              'use_rest_controller': False,
                              'use_sqlalchemy': True,
                              'use_webassets': True,
                              'version': '0.1'}

        create_project(blueberrypy_config)

        paths = ['',
                 '.cache',
                 'config',
                 'config/dev',
                 'config/dev/app.yml',
                 'config/dev/bundles.yml',
                 'config/dev/logging.yml',
                 'config/prod',
                 'config/prod/app.yml',
                 'config/prod/bundles.yml',
                 'config/prod/logging.yml',
                 'log',
                 'setup.py',
                 'src',
                 'src/blueberrytest',
                 'src/blueberrytest/__init__.py',
                 'src/blueberrytest/api.py',
                 'src/blueberrytest/controller.py',
                 'src/blueberrytest/model.py',
                 'src/blueberrytest/templates',
                 'src/blueberrytest/templates/404.html',
                 'src/blueberrytest/templates/base.html',
                 'src/blueberrytest/templates/index.html',
                 'src/tests',
                 'src/tests/__init__.py',
                 'src/tests/helper.py',
                 'src/tests/test_api.py',
                 'src/tests/test_controller.py',
                 'src/tests/test_model.py',
                 'static',
                 'static/css',
                 'static/css/screen.css',
                 'static/favicon.ico',
                 'static/img',
                 'static/js']

        for path in paths:
            self.assertTrue(os.path.exists(os.path.join(self.tmpdir, path)), "Path '%s' wasn't created" % path)




import os.path
import sys

from setuptools import setup, find_packages


install_requires = ['CherryPy>=3.2.2',
                    'Jinja2>=2.6',
                    'PyYAML>=3.10',
                    'python-dateutil<2.0',
                    'simplejson>=2.4.0']

dev_requires = ['Sphinx>=1.1.3',
                'nose>=1.1.2',
                'nose-testconfig>=0.8',
                'coverage>=3.5.1',
                'lazr.smtptest>=1.3',
                'ludibrio>=3.1.0',
                'tox>=1.3']

if sys.version_info[:2] < (2, 7):
    install_requires.append("argparse>=1.2.1")
    install_requires.append("logutils>=0.3.1")

readme_file = open(os.path.abspath(os.path.join(os.path.dirname(__file__), 'README.rst')), 'r')
readme = readme_file.read()
readme_file.close()

setup(name='blueberrypy',
      version='0.5.3',
      author='Jimmy Yuen Ho Wong',
      author_email='wyuenho@gmail.com',
      url='http://bitbucket.org/wyuenho/blueberrypy',
      description='CherryPy plugins and tools for integration with various libraries, including logging, Redis, SQLAlchemy and Jinja2 and webassets.',
      long_description=readme,
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Plugins',
                   'Environment :: Web Environment',
                   'Framework :: CherryPy',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Natural Language :: English',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7',
                   'Topic :: Database',
                   'Topic :: Internet :: WWW/HTTP :: Session',
                   'Topic :: Software Development :: Libraries',
                   'Topic :: Utilities'],
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      include_package_data=True,
      entry_points={'console_scripts': ['blueberrypy = blueberrypy.command:main']},
      zip_safe=False,
      install_requires=install_requires,
      extras_require={'speedups': ['cdecimal>=2.3',
                                   'MarkupSafe>=0.15',
                                   'hiredis>=0.1.0'],
                      'all': ['SQLAlchemy>=0.7.6',
                              'redis>=2.4.11',
                              'webassets>=0.6',
                              'Routes>=1.12.3',
                              'WebError>=0.10.3',
                              'Shapely>=1.2.14',
                              'GeoAlchemy>=0.7.1'],
                      'geospatial': ['Shapely>=1.2.14',
                                     'GeoAlchemy>=0.7.1'],
                      'dev': dev_requires})



import os
import sys
from code import InteractiveConsole


class Console(InteractiveConsole):

    def __init__(self, config):

        self.config = config

        try:
            import readline
        except ImportError, e:
            print(e)
        else:
            import rlcompleter

        startupfile = os.environ.get("PYTHONSTARTUP")
        if startupfile: execfile(startupfile, {}, {})

        proj_meta = config.project_metadata
        if proj_meta:
            package = proj_meta.get("package", None)

            sys.ps1 = "[%s]>>> " % package
            sys.ps2 = "[%s]... " % package

            self.prompt = package

        InteractiveConsole.__init__(self, locals=self.get_locals())

    def make_sqlalchemy_engine(self, prefix="sqlalchemy_engine"):

        config = self.config.sqlalchemy_config

        if prefix in config:
            section = config[prefix]
            from sqlalchemy.engine import engine_from_config
            return engine_from_config(section, '')
        else:
            engine_bindings = {}
            for section_name, section in config.iteritems():
                if section_name.startswith(prefix):
                    model_fqn = section_name[len(prefix) + 1:]
                    model_fqn_parts = model_fqn.rsplit('.', 1)
                    model_mod = __import__(model_fqn_parts[0], globals(), locals(), [model_fqn_parts[1]])
                    model = getattr(model_mod, model_fqn_parts[1])
                    engine_bindings[model] = engine_from_config(section)
            return engine_bindings

    def get_locals(self):
        import sys, os, os.path, time, datetime, pprint, inspect
        import sqlalchemy
        import sqlalchemy.orm
        import cherrypy
        import blueberrypy

        lcls = dict(locals())

        package_name = self.config.project_metadata and self.config.project_metadata["package"]
        if package_name:
            model = __import__(package_name + ".model", globals(), locals(), ["model"])
            if getattr(model, '__all__'):
                for name in model.__all__:
                    lcls[name] = getattr(model, name)
            else:
                for name, obj in vars(model).iteritems():
                    if not name.startswith("_"):
                        lcls[name] = obj

        if self.config.use_sqlalchemy:
            engine = self.make_sqlalchemy_engine()
            if isinstance(engine, dict):
                Session = sqlalchemy.orm.sessionmaker(twophase=True)
                Session.configure(binds=engine)
            else:
                Session = sqlalchemy.orm.sessionmaker()
                Session.configure(bind=engine)
            metadata = model.metadata
            metadata.bind = engine
            lcls['create_all'] = metadata.create_all
            lcls['drop_all'] = metadata.drop_all
            lcls['session'] = session = Session()
            session.bind.echo = True
            import atexit
            atexit.register(session.close)

        return lcls

    def raw_input(self, *args, **kw):
        try:
            r = InteractiveConsole.raw_input(self, *args, **kw)
            for encoding in (getattr(sys.stdin, 'encoding', None),
                             sys.getdefaultencoding(), 'utf-8', 'latin-1'):
                if encoding:
                    try:
                        return r.decode(encoding)
                    except UnicodeError:
                        pass
                    return r
        except EOFError:
            self.write(os.linesep)
            session = self.locals.get("session")
            if session is not None and \
                session.new or \
                session.dirty or \
                session.deleted:

                r = raw_input("Do you wish to commit your "
                              "database changes? [Y/n]")
                if not r.lower().startswith("n"):
                    self.push("session.commit()")
            raise



from jinja2 import Environment as Jinja2Environment

from blueberrypy.exc import BlueberryPyNotConfiguredError

__all__ = ["jinja2_env", "configure_jinja2", "get_template"]


jinja2_env = None


def configure_jinja2(assets_env=None, **kwargs):
    global jinja2_env

    autoescape = kwargs.pop("autoescape", False)
    extensions = kwargs.pop("extensions", [])

    if assets_env:
        from webassets.ext.jinja2 import AssetsExtension
        extensions.append(AssetsExtension)

    if autoescape:
        autoescape = lambda filename: filename.rsplit('.', 1)[1] in ('html', 'xml', 'htm') if isinstance(filename, basestring) else False
        extensions.append("jinja2.ext.autoescape")

    jinja2_env = Jinja2Environment(autoescape=autoescape, extensions=extensions,
                                  **kwargs)

    if assets_env:
        jinja2_env.assets_environment = assets_env

    return jinja2_env

def get_template(*args, **kwargs):
    if not jinja2_env:
        raise BlueberryPyNotConfiguredError("Jinja2 not configured")
    return jinja2_env.get_template(*args, **kwargs)



import cherrypy

from blueberrypy.plugins import SQLAlchemyPlugin

class EngineTest(object):

    def engine(self):
        return str(cherrypy.engine.sqlalchemy.engine)
    engine.exposed = True

    def exit(self):
        # This handler might be called before the engine is STARTED if an
        # HTTP worker thread handles it before the HTTP server returns
        # control to engine.start. We avoid that race condition here
        # by waiting for the Bus to be STARTED.
        cherrypy.engine.wait(state=cherrypy.engine.states.STARTED)
        cherrypy.engine.exit()
    exit.exposed = True


def log_test_case_name():
    if cherrypy.config.get('test_case_name', False):
        cherrypy.log("STARTED FROM: %s" % cherrypy.config.get('test_case_name'))
cherrypy.engine.subscribe('start', log_test_case_name, priority=6)


saconf = {'sqlalchemy_engine': {'url': 'sqlite://'}}
cherrypy.engine.sqlalchemy = SQLAlchemyPlugin(cherrypy.engine, saconf)
cherrypy.config.update({'environment': 'test_suite',
                        'engine.sqlalchemy.on': True})
cherrypy.tree.mount(EngineTest())



import cPickle as pickle
import logging
import threading
import time

from pprint import pformat

from cherrypy.lib import sessions
from redis import StrictRedis as _RedisClient


__all__ = ["RedisSession"]


logger = logging.getLogger(__name__)


def normalize_sep(prefix, sep=':'):

    prefix = str(prefix)

    if not prefix.endswith(sep):
        return prefix + sep
    else:
        return prefix.rstrip(':') + ':'


class RedisSession(sessions.Session):

    locks = {}

    prefix = "cp-session:"

    debug = False

    @classmethod
    def setup(cls, **kwargs):

        cls.prefix = normalize_sep(kwargs.pop("prefix", cls.prefix))

        for k, v in kwargs.items():
            setattr(cls, k, v)
        cls.cache = cache = _RedisClient(**kwargs)
        redis_info = cache.info()
        if cls.debug:
            logger.info("Redis server ready.\n%s" % pformat(redis_info))
        else:
            logger.info("Redis server ready.")

    def _exists(self):
        return self.cache.exists(self.prefix + self.id)

    def _load(self):
        data = self.cache.get(self.prefix + self.id)
        if data:
            retval = pickle.loads(data)
            return retval

    def _save(self, expiration_time):
        key = self.prefix + self.id
        td = int(time.mktime(expiration_time.timetuple()))

        data = pickle.dumps((self._data, expiration_time),
                            pickle.HIGHEST_PROTOCOL)

        def critical_section(pipe):
            pipe.multi()
            pipe.set(key, data)
            pipe.expireat(key, td)

        self.cache.transaction(critical_section, key)

    def _delete(self):
        self.cache.delete(self.prefix + self.id)

    def acquire_lock(self):
        self.locked = True
        self.locks.setdefault(self.id, threading.RLock()).acquire()

    def release_lock(self):
        self.locks[self.id].release()
        self.locked = False

    def __len__(self):
        """Return the number of active sessions."""
        keys = self.cache.keys(self.prefix + '*')
        return len(keys)
